package mvd.jester;

import java.io.IOException;
import java.time.LocalTime;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ObjectArrays;
import com.google.common.collect.Table;
import com.google.common.collect.TreeBasedTable;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import mvd.jester.tests.AbstractTest;
import mvd.jester.utils.Logger;

/**
 * ResultCollector
 */
public class ResultLogger {

    private final long numberOfProcessors;

    public ResultLogger(long numberOfProcessors) {
        this.numberOfProcessors = numberOfProcessors;
    }

    // public void logAll() {
    // if (!testResults.isEmpty()) {
    // logFeasibility(testResults, "test");
    // logFeasibilityRatio(testResults, "test");
    // logTardinessStatistics(testResults, "test");
    // logFailedTardiness(testResults, "test");
    // }
    // if (!simulatorResults.isEmpty()) {
    // logFeasibility(simulatorResults, "sim");
    // logFeasibilityRatio(simulatorResults, "sim");
    // logTardinessStatistics(simulatorResults, "sim");
    // logFailedTardiness(simulatorResults, "sim");
    // }
    // }

    public void logTests(Set<ResultCollector<AbstractTest>> results) {
        logFeasibility(results, "test");
        logTaskRatio(results, "test");
    }

    public <T extends TypeInterface> void logFeasibility(Set<ResultCollector<T>> results,
            String type) {
        LocalTime date = LocalTime.now();
        Logger log = new Logger("./results/feasibility_" + type + "_" + numberOfProcessors + "_"
                + date.getHour() + ":" + date.getMinute() + ".txt");

        Table<Long, ResultCollector<T>, Long> resultTable = TreeBasedTable.create();
        Set<ResultCollector<T>> resultCollectors = new TreeSet<>();

        for (ResultCollector<T> r : results) {
            System.out.println(r.getName() + ": "
                    + r.getResults().stream().filter(p -> p.checkTasksetFeasible()).count());
        }

        for (long util = 0; util < numberOfProcessors * 10; util++) {
            for (ResultCollector<T> rc : results) {
                resultCollectors.add(rc);
                final long local_util = util;
                long feasibleTasksets = rc.getResults().stream()
                        .filter(r -> Math.round(r.getUtilization() * 10) == local_util)
                        .filter(r -> r.checkTasksetFeasible()).count();
                resultTable.put(util, rc, feasibleTasksets);
            }
        }

        logData(log, resultTable, resultCollectors);
    }

    public <T extends TypeInterface> void logTaskRatio(Set<ResultCollector<T>> results,
            String type) {
        LocalTime date = LocalTime.now();
        Logger log = new Logger("./results/task_ratio_" + type + "_" + numberOfProcessors + "_"
                + date.getHour() + ":" + date.getMinute() + ".txt");

        Table<Long, ResultCollector<T>, Long> resultTable = TreeBasedTable.create();
        Set<ResultCollector<T>> resultCollectors = new TreeSet<>();

        for (long ratio = 0; ratio < 10; ratio += 1) {
            for (ResultCollector<T> rc : results) {
                resultCollectors.add(rc);
                final long local_ratio = ratio;
                long feasibleTasksets = rc.getResults().stream()
                        .filter(r -> Math.round(r.getParallelTaskRatio() * 10) == local_ratio)
                        .filter(r -> r.checkTasksetFeasible()).count();
                resultTable.put(ratio, rc, feasibleTasksets);
            }
        }

        logData(log, resultTable, resultCollectors);
    }

    private <T extends TypeInterface> void logData(Logger log,
            Table<Long, ResultCollector<T>, Long> resultTable,
            Set<ResultCollector<T>> resultCollectors) {
        final Appendable out = new StringBuilder();
        try {
            String[] resultCollectorNames = resultCollectors.stream()
                    .map(ResultCollector<T>::getName).toArray(String[]::new);
            String[] header = ObjectArrays.concat("Utilization", resultCollectorNames);
            final CSVPrinter printer = CSVFormat.DEFAULT.withHeader(header).print(out);

            printer.printRecords(resultTable.rowMap().entrySet().stream()
                    .map(entry -> ImmutableList.builder().add((double) entry.getKey() / 10)
                            .addAll(entry.getValue().values()).build())
                    .collect(Collectors.toList()));
        } catch (final IOException e) {
            e.printStackTrace();
        }

        log.log(out);
        log.finalize();
    }


    // public <T extends TypeInterface> void logFeasibility(
    // Table<Long, ResultCollector<T>, Set<SchedulingInfo>> results, String type) {
    // LocalTime date = LocalTime.now();
    // Logger log = new Logger("./results/feasibility_" + type + "_" + numberOfProcessors + "_"
    // + date.getHour() + ":" + date.getMinute() + ".txt");
    // String firstLine = new String("Utilization");

    // if (!results.isEmpty()) {
    // for (ResultCollector<T> pair : results.columnKeySet()) {
    // firstLine = firstLine + "\t" + pair.getName();
    // }

    // log.log(firstLine);

    // for (Long util : totalNumberOfTasksets.keySet()) {
    // String line = String.valueOf((double) util / 10);
    // for (ResultCollector<T> pair : results.columnKeySet()) {
    // long feasibleTasksets = results.get(util, pair).stream()
    // .filter(s -> s.checkTasksetFeasible()).count();
    // line += "\t" + feasibleTasksets;
    // }
    // log.log(line);
    // }
    // }

    // log.finalize();
    // }


    // public <T extends TypeInterface> void logFailedTardiness(
    // Table<Long, ResultCollector<T>, Set<SchedulingInfo>> results, String type) {
    // LocalTime date = LocalTime.now();
    // Logger log = new Logger("./results/failed_tardiness_" + type + "_" + numberOfProcessors
    // + "_" + date.getHour() + ":" + date.getMinute() + ".txt");
    // String firstLine = new String("Utilization");

    // if (!results.isEmpty()) {
    // for (ResultCollector<T> pair : results.columnKeySet()) {
    // firstLine = firstLine + "\t" + pair.getName();
    // }

    // log.log(firstLine);

    // for (Long kv : totalNumberOfTasksets.keySet()) {
    // String line = String.valueOf((double) kv / 10);
    // for (ResultCollector<T> pair : results.columnKeySet()) {
    // Set<SchedulingInfo> simulationInfos = results.get(kv, pair);
    // List<Long> values = new ArrayList<>();

    // for (SchedulingInfo s : simulationInfos) {
    // Optional<TerminationInfo> failedTerminationInfo =
    // s.getFailedTerminationInfo();
    // if (failedTerminationInfo.isPresent()) {
    // values.add(failedTerminationInfo.get().getLateness());
    // }
    // }
    // double meanTardiness = 0;
    // if (!values.isEmpty()) {
    // meanTardiness = Stats.meanOf(values.iterator());
    // }
    // line += "\t" + meanTardiness;
    // }
    // log.log(line);
    // }
    // }

    // log.finalize();
    // }

    // public <T extends TypeInterface> void logTardinessStatistics(
    // Table<Long, ResultCollector<T>, Set<SchedulingInfo>> results, String type) {
    // LocalTime date = LocalTime.now();
    // Logger log = new Logger("./results/statistics_tardiness_" + type + "_" + numberOfProcessors
    // + "_" + date.getHour() + ":" + date.getMinute() + ".txt");
    // String firstLine = new String("Utilization");

    // if (!results.isEmpty()) {
    // for (ResultCollector<T> pair : results.columnKeySet()) {
    // firstLine = firstLine + "\t" + pair.getName();
    // }

    // log.log(firstLine);

    // for (Long kv : totalNumberOfTasksets.keySet()) {
    // String line = String.valueOf((double) kv / 10);
    // for (ResultCollector<T> pair : results.columnKeySet()) {
    // Set<SchedulingInfo> simulationInfos = results.get(kv, pair);
    // DescriptiveStatistics stats = new DescriptiveStatistics();

    // for (SchedulingInfo s : simulationInfos) {
    // for (TerminationInfo t : s.getTerminationInfos()) {
    // stats.addValue(t.getLateness());
    // }
    // }

    // line += "\t" + stats.getMean();
    // }
    // log.log(line);
    // }
    // }

    // log.finalize();
    // }

}
